'use strict';

var angular = require('angular');
require('../modules');

var module = angular.module('info.services');

/**
 * 信息发布列表页缓存处理函数,只处理第一页的数据
 */
module.factory('InfoListCache', function() {
  function InfoListCache(infoListId, length) {
    this.infoListId = infoListId;
    this.length = length;
    this.dataCacheKey = 'infolist_data_cache_' + this.infoListId;
    this.timestampCacheKey = 'infolist_timestamp_cache_' + this.infoListId;
    this.dataTotalElementsKey = 'infolist_totalElements_cache_' + this.infoListId;
  }

  InfoListCache.prototype = {
    update: function(timestamp, data, totalElements) {
      localStorage.setItem(this.timestampCacheKey, timestamp);
      localStorage.setItem(this.dataTotalElementsKey, totalElements > this.getTotalElements() ? totalElements : this.getTotalElements());
      var cachedData = this.getData();

      var result = data.concat(cachedData || []).slice(0, this.length);
      localStorage.setItem(this.dataCacheKey, JSON.stringify(result));
      return result;
    },
    getData: function() {
      try {
        return JSON.parse(localStorage.getItem(this.dataCacheKey));
      } catch (e) {
        console.error(e);
      }
    },
    getTimestamp: function() {
      return localStorage.getItem(this.timestampCacheKey) || -1;
    },
    getTotalElements: function() {
      return parseInt(JSON.parse(localStorage.getItem(this.dataTotalElementsKey))) || 0;
    }
  };

  return InfoListCache;
});