'use strict';

var angular = require('angular');
require('./modules');
require('./services/VPNManagerService');
require('../appInitFlow/services/appInitFlow');
require('./services/vpnService');

var module = angular.module('vpnmanager');

module.config(function($httpProvider) {
  $httpProvider.interceptors.push('waitLoginVPNHttpRequestInterceptor');
});

function anyPromise($q, $timeout, promise, timeoutPromise) {
  var defer = $q.defer();
  var deferred = false;
  var resolve = function(data) {
    if (deferred) {
      return;
    }
    deferred = true;
    defer.resolve(data);
    $timeout.cancel(timeoutPromise);
  }

  promise.then(resolve, resolve);
  timeoutPromise.then(resolve, resolve);

  return defer.promise;
}

module.factory('waitLoginVPNHttpRequestInterceptor', function($injector) {
  var waitLoginVPNPromiseCache = {};
  function waitLoginVPN() {
    var applicationConfig = $injector.get('CoreAppConfig').applicationConfig;
    var vpnServer = applicationConfig.getKeyValue('vpnServer');
    var $timeout = $injector.get('$timeout'),
      $q = $injector.get('$q');

    if (vpnServer) {
      if (!waitLoginVPNPromiseCache[vpnServer]) {
        var timeoutPromise = $timeout(function() {return true;}, ionic.Platform.isAndroid() ? 10 : 5);
        var promise = anyPromise($q, $timeout, $injector.get('vpnService').waitLoginVpn(), timeoutPromise);
        waitLoginVPNPromiseCache[vpnServer] = promise;
      }

      return waitLoginVPNPromiseCache[vpnServer];
    } else {
      return $q.when(true);
    }
  }

  return {
    request: function(config) {
      if (config.url.indexOf('views') === -1) {
        var applicationConfig = $injector.get('CoreAppConfig').applicationConfig;
        if (!applicationConfig.getServerUrl()) {
          return applicationConfig.getKeyValueUntilNoEmpty('serverUrl')
          .then(function() {
            return waitLoginVPN();
          }).then(function() { return config; });
        } else {
            return waitLoginVPN().then(function() { return config; });
        }
      }
      return config;
    }
  };
});

module.exports = module;
